@@ -10,6 +10,8 @@ from einops import rearrange, repeat
 import alphafold2_pytorch.constants as constants
 from alphafold2_pytorch.reversible import ReversibleSequence, SequentialSequence
 
+from se3_transformer_pytorch import SE3Transformer
+
 # helpers
 
 def exists(val):
@@ -25,13 +27,25 @@ def default(val, d):
 class PreNorm(nn.Module):
     def __init__(self, dim, fn):
         super().__init__()
-        self.norm = nn.LayerNorm(dim)
         self.fn = fn
+        self.norm = nn.LayerNorm(dim)
 
     def forward(self, x, *args, **kwargs):
         x = self.norm(x)
         return self.fn(x, *args, **kwargs)
 
+class PreNormCross(nn.Module):
+    def __init__(self, dim, fn):
+        super().__init__()
+        self.fn = fn
+        self.norm = nn.LayerNorm(dim)
+        self.norm_context = nn.LayerNorm(dim)
+
+    def forward(self, x, context, *args, **kwargs):
+        x = self.norm(x)
+        context = self.norm(context)
+        return self.fn(x, context, *args, **kwargs)
+
 class GEGLU(nn.Module):
     def forward(self, x):
         x, gates = x.chunk(2, dim = -1)
@@ -170,22 +184,28 @@ class Alphafold2(nn.Module):
 
         # main trunk modules
 
-        wrapper = partial(PreNorm, dim)
+        prenorm = partial(PreNorm, dim)
+        prenorm_cross = partial(PreNormCross, dim)
 
         layers = nn.ModuleList([])
         for _ in range(depth):
-            layers.append(nn.ModuleList([
-                wrapper(AxialAttention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
-                wrapper(FeedForward(dim = dim, dropout = ff_dropout)),
-                wrapper(Attention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
-                wrapper(FeedForward(dim = dim, dropout = ff_dropout)),
-            ]))
+
+            # self attention, for both main sequence and msa
 
             layers.append(nn.ModuleList([
-                wrapper(Attention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
-                wrapper(FeedForward(dim = dim, dropout = ff_dropout)),
-                wrapper(Attention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
-                wrapper(FeedForward(dim = dim, dropout = ff_dropout)),
+                prenorm(AxialAttention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
+                prenorm(FeedForward(dim = dim, dropout = ff_dropout)),
+                prenorm(Attention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
+                prenorm(FeedForward(dim = dim, dropout = ff_dropout)),
+            ]))
+
+            # cross attention, for main sequence -> msa and then msa -> sequence
+
+            layers.append(nn.ModuleList([
+                prenorm_cross(Attention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
+                prenorm(FeedForward(dim = dim, dropout = ff_dropout)),
+                prenorm_cross(Attention(dim = dim, heads = heads, dim_head = dim_head, dropout = attn_dropout)),
+                prenorm(FeedForward(dim = dim, dropout = ff_dropout)),
             ]))
 
         if not reversible:
@@ -243,6 +263,12 @@ class Alphafold2(nn.Module):
 
         x, m = self.net(x, m, mask = x_mask, msa_mask = msa_mask)
 
+        # structural refinement
+
+        ### TODO - use SE3Transformer here, as details emerge about the iterative refinement, fill-in here
+
+        # final out, do alphafold1's distogram for now
+
         x = rearrange(x, 'b (h w) d -> b h w d', h = n)
         x = (x + rearrange(x, 'b i j d -> b j i d')) * 0.5  # symmetrize
         return self.to_distogram_logits(x)
