@@ -17,11 +17,12 @@ except:
     GLOBAL_PAD_CHAR = 0
 
 # own
-from alphafold2_pytorch.alphafold2 import DISTOGRAM_BUCKETS
+
+import alphafold2_pytorch.constants as constants
 
 # constants: same as in alphafold2.py
 
-DISTANCE_THRESHOLDS = torch.linspace(2, 20, steps = DISTOGRAM_BUCKETS)
+DISTANCE_THRESHOLDS = torch.linspace(2, 20, steps = constants.DISTOGRAM_BUCKETS)
 
 # decorators
 
@@ -31,7 +32,6 @@ def set_backend_kwarg(fn):
         if backend == 'auto':
             backend = 'torch' if isinstance(args[0], torch.Tensor) else 'numpy'
         kwargs.update(backend = backend)
-        print('settinb backend kwargs', backend, kwargs)
         return fn(*args, **kwargs)
     return inner
 
@@ -78,7 +78,7 @@ def download_pdb(name, route):
         * route: str. route of the destin file. usually ".pdb" extension
         Output: route of destin file
     """
-    os.system("curl https://files.rcsb.org/download/{0}.pdb > {1}".format(name, route))
+    os.system(f"curl https://files.rcsb.org/download/{name}.pdb > {route}")
     return route
 
 def clean_pdb(name, route=None, chain_num=None):
@@ -100,7 +100,7 @@ def clean_pdb(name, route=None, chain_num=None):
             if chain_num != chain.index:
                 continue
         # select indexes of chain
-        chain_idxs = raw_prot.topology.select("chainid == {0}".format(chain.index))
+        chain_idxs = raw_prot.topology.select(f"chainid == {str(chain.index)}")
         idxs.extend( chain_idxs.tolist() )
     # sort: topology and xyz selection are ordered
     idxs = sorted(idxs)
@@ -128,7 +128,7 @@ def custom2pdb(coords, proteinnet_id, route):
         coords = coords.T
     coords = np.newaxis(coords, axis=0)
     # get pdb id and chain num
-    pdb_name, china_num = proteinnet_id.split("#")[-1].split("_")[:-1]
+    pdb_name, chain_num = proteinnet_id.split("#")[-1].split("_")[:-1]
     pdb_destin = "/".join(route.split("/")[:-1])+"/"+pdb_name+".pdb"
     # download pdb file and select appropiate 
     download_pdb(pdb_name, pdb_destin)
@@ -227,7 +227,7 @@ def center_distogram_torch(distogram, bins=DISTANCE_THRESHOLDS, min_t=1., center
     """
     shape, device = distogram.shape, distogram.device
     # threshold to weights and find mean value of each bin
-    n_bins = bins - 0.5 * (bins[2] - bins[2])
+    n_bins = bins - 0.5 * (bins[2] - bins[1])
     n_bins[0]  = 1.5
     # TODO: adapt so that mean option considers IGNORE_INDEX
     n_bins[-1] = n_bins[-1]
@@ -237,7 +237,7 @@ def center_distogram_torch(distogram, bins=DISTANCE_THRESHOLDS, min_t=1., center
         cum_dist = torch.cumsum(distogram, dim=-1)
         medium   = 0.5 * torch.ones(*cum_dist.shape[:-1], device=device).unsqueeze(dim=-1)
         central  = torch.searchsorted(cum_dist, medium).squeeze()
-        central  = n_bins[ torch.minimum(central, torch.tensor(DISTOGRAM_BUCKETS-1)).long() ]
+        central  = n_bins[ torch.minimum(central, torch.tensor( len(n_bins)-1 )).long() ].unsqueeze(dim=0)
     elif center == "mean":
         central  = (distogram * n_bins).sum(dim=-1)
     # create mask for last class - (IGNORE_INDEX)   
@@ -573,7 +573,7 @@ def MDScaling(pre_dist_mat, **kwargs):
         Assumes (for now) distrogram is (N x N) and symmetric.
         For support of ditograms: see `center_distogram_torch()`
         Inputs:
-        * distogram: (1, N, N) distance matrix.
+        * pre_dist_mat: (1, N, N) distance matrix.
         * weights: optional. (N x N) pairwise relative weights .
         * iters: number of iterations to run the algorithm on
         * tol: relative tolerance at which to stop the algorithm if no better
@@ -644,4 +644,15 @@ def GDT(A, B, *, mode="TS", cutoffs=[1,2,4,8], weights=None):
 @set_backend_kwarg
 @invoke_torch_or_numpy(tmscore_torch, tmscore_numpy)
 def TMscore(A, B):
+    """ Returns TMscore as defined here (higher is better):
+        >0.5 (likely) >0.6 (highly likely) same folding. 
+        = 0.2. https://en.wikipedia.org/wiki/Template_modeling_score
+        Warning! It's not exactly the code in:
+        https://zhanglab.ccmb.med.umich.edu/TM-score/TMscore.cpp
+        but will suffice for now. 
+        Inputs: 
+            * A,B are (B x 3 x N) (np.array or torch.tensor)
+            * mode: one of ["numpy", "torch", "auto"] for backend
+        Outputs: tensor/array of size (B,)
+    """
     return A, B
